/*
*******************************************************************************
* Copyright (c) 2021 by M5Stack
*                  Equipped with Atom-Lite/Matrix sample source code
*                          配套  Atom-Lite/Matrix 示例源代码
* Visit the website for more information：https://docs.m5stack.com/en/atom/atom_can
* 获取更多资料请访问：https://docs.m5stack.com/zh_CN/atom/atom_can
*
* describe: Can FOR SAM EV2
* modified by Thomas Lengyel
* date：2023/06/01
* Version: B0.1
*******************************************************************************
*/


#include "M5Atom.h"
#include "ESP32CAN.h"
#include "CAN_config.h"
#include "retroTerm.h"


#define CAN_UNIT
//#define CAN_SENDER

#define CAN_MSG_ID      0x001

CAN_device_t CAN_cfg;             // CAN Config.  CAN配置。
unsigned long previousMillis = 0; // will store last time a CAN Message was send.  将存储上次发送CAN消息的时间。
const int   interval = 80;        // interval at which send CAN Messages (milliseconds).  发送CAN消息的间隔(毫秒)。
const int   rx_queue_size = 10;     // Receive Queue size.  接收队列大小
int         LedNumberCount = 0;
int         waitCANData = 0;

uint8_t     msgcnt = 0;

retroTerm terminal;       //Create a terminal instance


void setup()
{
    M5.begin(true,true,true);
    terminal.begin(Serial);
    terminal.reset();             //Resets the terminal, clearing colours etc.
    terminal.hideCursor();        //Hide the cursor
    terminal.printAt(1,1," Atom install OK");  //Print the title at the top of the page
    
    M5.dis.fillpix(CRGB(0,100,0));

    CAN_cfg.speed = CAN_SPEED_125KBPS;

    //CAN_cfg.tx_pin_id = GPIO_NUM_26;
    //CAN_cfg.rx_pin_id = GPIO_NUM_32;

    CAN_cfg.tx_pin_id = GPIO_NUM_22;
    CAN_cfg.rx_pin_id = GPIO_NUM_19;
    CAN_cfg.rx_queue = xQueueCreate(rx_queue_size, sizeof(CAN_frame_t));

    ESP32Can.CANInit();
    terminal.printAt(1,1," CAN install OK");  //Print the title at the top of the page
    
    
    M5.dis.fillpix(CRGB(100,0,0));
}

void print_diff(int ex, int ey,CAN_frame_t print_frame, CAN_frame_t frames[])
{
  char buff[128];



  sprintf(buff,"%d DIFF %02X %02X %02X %02X %02X %02X %02X %02X : %02X %02X %02X %02X %02X %02X %02X %02X ",ey-2,frames[ey-2].data.u8[0],frames[ey-2].data.u8[1],frames[ey-2].data.u8[2],frames[ey-2].data.u8[3],frames[ey-2].data.u8[4],frames[ey-2].data.u8[5],frames[ey-2].data.u8[6],frames[ey-2].data.u8[7],print_frame.data.u8[0],print_frame.data.u8[1],print_frame.data.u8[2],print_frame.data.u8[3],print_frame.data.u8[4],print_frame.data.u8[5],print_frame.data.u8[6],print_frame.data.u8[7]);

  //terminal.printAt(ex,ey,buff);
  
  /*
  for (int i = 0; i < print_frame.FIR.B.DLC; i++)
            {
                terminal.print("0x%02X ", print_frame.data.u8[i]);
            }
            terminal.print("\r\n");
  */



  
}

void loop()
{
    unsigned long currentMillis = millis();

#ifndef CAN_SENDER
    // Send CAN Message
    CAN_frame_t rx_frame;
    int myvalue = 0;
    CAN_frame_t my_frames[200];

    if (xQueueReceive(CAN_cfg.rx_queue, &rx_frame, 3 * portTICK_PERIOD_MS) == pdTRUE)
    {

        
        

        if (rx_frame.FIR.B.RTR == CAN_RTR)
        {
            //terminal.print(" RTR from 0x%08X, DLC %d\r\n", rx_frame.MsgID, rx_frame.FIR.B.DLC);
        }
        else
        {


            switch(rx_frame.MsgID)
            {
              case 0x0406:    // New standard frame from 0x00000406, DLC 5, Data 0x00 0x00 0x09 0xBF 0x0E
                  //if(my_frames[0].data.u64 == rx_frame.data.u64)
                  //{
                  //  }else{
                  my_frames[rx_frame.data.u8[2]]=rx_frame;
                  terminal.printAt(1,50,rx_frame.data.u8[2]);
                  
                  print_diff(1,rx_frame.data.u8[2],rx_frame,my_frames);
                  myvalue = rx_frame.data.u8[4]<< 8;
                  myvalue = myvalue | rx_frame.data.u8[3];
                  terminal.printAt(1,rx_frame.data.u8[2],rx_frame.data.u8[2]);
                  
                  terminal.printAt(4,rx_frame.data.u8[2],myvalue);
                  //print_diff(1,2,rx_frame,my_frames);
                  //}
                  break;
              
                    
              default:
                  print_diff(1,20,rx_frame,my_frames);
                break;
              
            }
          
            
            
        }
    }
#else

#endif
    //waitCANData
    M5.update();
    if(M5.Btn.isPressed())
    {
        M5.dis.clear();
        esp_restart();
    }
    delay(10);
}
